"""Add leased status to parcel.

Revision ID: 75aee9c50aa
Revises: 1654373c7899
Create Date: 2015-03-29 17:48:45.181684

"""

# revision identifiers, used by Alembic.
from alembic import op
import sqlalchemy as sa

revision = '75aee9c50aa'
down_revision = '1654373c7899'


def upgrade():
    op.add_column(
        'parcel',
        sa.Column('leased', sa.Boolean(), nullable=False,
                  server_default='False'))
    op.execute(
        """UPDATE parcel SET leased = true WHERE number in
           (70, 74, 109, 148, 111, 89, 160, 135, 136, 84, 103, 133, 138,
            165, 100, 99, 121, 122);""")


def downgrade():
    op.drop_column('parcel', 'leased')



# encoding=utf8
from __future__ import unicode_literals
import pyramid.testing
import pyramid_mailer
import pytest
import sw.allotmentclub.browser.app
import sw.allotmentclub.browser.testing
import sw.allotmentclub.conftest


DEFAULT_SETTINGS = {
    'beaker.lock_dir': '/tmp',
    'https_disabled': 'true',
    'public_server_name': 'localhost',
    'pyramid.default_locale_name': 'de_DE',
    'pyramid_deform.tempdir': '/tmp',
    'pyramid.render_traceback': 'true',
    'contact_default_recipients': 'vorstand@roter-see.de',
    'bitbtl.secret': 'secret',
    'bitbtl.quality': 5,
}


@pytest.fixture(scope='session')
def portal_app(database_session):
    """Create a portal app."""
    return sw.allotmentclub.browser.app.Portal(
        testing=True, **DEFAULT_SETTINGS)


@pytest.fixture(scope='session')
def wsgi_app(portal_app):
    """Create the congress WSGI app."""
    return portal_app({})


@pytest.fixture(scope='function')
def browser(wsgi_app, verwalter):
    """Get a zope.testbrowser for the WSGI-APP."""
    return sw.allotmentclub.browser.testing.Browser(wsgi_app=wsgi_app)


@pytest.fixture(scope='function')
def dummy_request(portal_app, wsgi_app):
    """Get a request with the current app."""
    registry = portal_app.config.registry
    request = pyramid.testing.DummyRequest(_registry=registry)
    context = pyramid.threadlocal.manager.get().copy()
    context['request'] = request
    context['registry'] = registry
    pyramid.threadlocal.manager.push(context)
    return pyramid.testing.DummyRequest(_registry=registry)


@pytest.fixture(scope='function')
def mailer(dummy_request):
    """Get the pyramid_mailer testing mailer."""
    mailer = pyramid_mailer.get_mailer(dummy_request)
    mailer.outbox = []
    return mailer



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .model import Object
from .base import format_eur
from sqlalchemy import Column, Integer, Float, String, ForeignKey, DateTime
import sqlalchemy.orm


class Assignment(Object):
    """ Der Arbeitseinsatz. """

    purpose = Column(String(254))
    responsible_id = Column(Integer, ForeignKey('member.id'), nullable=True)
    responsible = sqlalchemy.orm.relation(
        'Member', uselist=False, backref='responsible_for')
    day = Column(DateTime)
    accounting_year = Column(Integer)
    attendees = sqlalchemy.orm.relation(
        'AssignmentAttendee', uselist=True, backref='assignment',
        cascade='all,delete')


class AssignmentAttendee(Object):
    """ Teilnehmer an einem Arbeitseinsatz."""

    assignment_id = Column(
        Integer, ForeignKey('assignment.id'), nullable=False)
    member_id = Column(
        Integer, ForeignKey('member.id'), nullable=True)
    member = sqlalchemy.orm.relation(
        'Member', uselist=False, backref='assignments')
    hours = Column(Float(1), nullable=True)


def get_assignment_mail_data(member, year):
    if member.leaving_year or not member.allotments:
        return
    price = int((5 - member.assignment_hours) * 10) * 10000
    if price <= 0:
        return
    content_data = dict(
        assignment_hours=member.assignment_hours,
        to_pay=format_eur(price),
        year=year,
        iban=member.iban,
        bic=member.bic,
        direct_debit=member.direct_debit)
    subject = u'Fehlende Arbeitsstunden %s' % year
    yield subject, content_data



from __future__ import unicode_literals
from babel.dates import format_datetime
from pyramid.view import view_config
from sw.allotmentclub import Log, User
import sw.allotmentclub.browser.base
import datetime
import hashlib


class Query(sw.allotmentclub.browser.base.Query):

    def select(self):
        return (
            self.db.query(
                Log.created.label('Datum'),
                Log.user_id.label('Benutzer'),
                Log.name.label('Typ'),
                Log.msg.label('Nachricht'))
            .select_from(Log)
            .filter(Log.created >= (
                datetime.datetime.now() - datetime.timedelta(days=14)))
            .filter(Log.user_id != 1))


def format_date(dt):
    now = datetime.datetime.now()
    if dt > now - datetime.timedelta(minutes=15):
        return 'gerade eben'
    if dt.date() == now.date():
        return 'heute'
    if dt.date() == (now - datetime.timedelta(days=1)).date():
        return 'gestern'
    return format_datetime(dt, "EEEE, d.M.Y", locale='de_DE'),


@view_config(route_name='home', renderer='json', permission='view')
class HomeView(sw.allotmentclub.browser.base.TableView):

    query_class = Query
    default_order_by = 'created'
    default_sort_direction = 'desc'

    def __call__(self):
        result = super(HomeView, self).__call__()
        last = None
        timeline = []
        for date, user_id, type_, msg in sw.allotmentclub.json_result(
                result['data']['data']):
            if timeline:
                last = timeline[-1]
            time = format_date(date)
            user = User.get(user_id)
            username = user.username
            if last and last['username'] == username and last['time'] == time:
                if msg == last['detail'][-1]:
                    continue
                last['detail'].append(msg)
            else:
                item = {
                    'time': time,
                    'username': username,
                    'firstname': user.vorname,
                    'lastname': user.nachname,
                    'detail': [msg]
                }
                if username == 'system':
                    item['fa_icon'] = 'fa-gear'
                else:
                    item['gravatar_url'] = (
                        'https://www.gravatar.com/avatar/%s' % (
                            hashlib.md5(
                                user.email.encode('utf-8')).hexdigest()))
                timeline.append(item)
        result['data']['timeline'] = timeline
        return result



"""re #21: Add tap_water information

Revision ID: 708df911eb9e
Revises: f86ea5808406
Create Date: 2016-07-31 12:32:27.573230

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '708df911eb9e'
down_revision = 'f86ea5808406'


def upgrade():
    op.add_column('parcel',
                  sa.Column('tap_water', sa.Boolean(), nullable=False,
                            server_default='False'))
    op.execute(
        """UPDATE parcel SET tap_water = true WHERE number in
           (SELECT p.number
            FROM parcel as p JOIN allotment al on p.allotment_id = al.id
            WHERE al.number in (
                104, 106, 108, 110, 112, 114, 120, 122, 126, 130, 138, 205,
                210, 211, 214, 217, 219, 220, 237, 239, 241, 247, 261, 307,
                311, 312, 314, 310, 401, 403, 409, 414, 417, 424, 128, 209,
                229, 235, 252, 253, 309, 315, 335, 337, 411, 415, 321, 330,
                333, 329, 320, 339));""")
    op.execute(
        """INSERT into accessauthority (viewname, user_id, organization_id)
           VALUES ('member_list_leased', 4, 1)""")
    op.execute(
        """INSERT into accessauthority (viewname, user_id, organization_id)
           VALUES ('member_list_leased', 2, 1)""")
    op.execute(
        """INSERT into accessauthority (viewname, user_id, organization_id)
           VALUES ('member_list_tap_water', 4, 1)""")
    op.execute(
        """INSERT into accessauthority (viewname, user_id, organization_id)
        VALUES ('member_list_tap_water', 2, 1)""")


def downgrade():
    op.drop_column('parcel', 'tap_water')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from ..log import user_data_log, log_with_user
from .base import date_time, to_string, string_agg, get_selected_year
from .base import datetime_now  # noqa
from sqlalchemy import func
from pyramid.view import view_config
from sw.allotmentclub import Assignment, AssignmentAttendee, Member, Allotment
import collections
import sw.allotmentclub.browser.base


class Query(sw.allotmentclub.browser.base.Query):

    formatters = {
        'Datum': date_time,
    }

    data_class = {
        'Datum': 'expand'
    }
    data_hide = {
        'Zweck': 'phone,tablet',
        'Verantwortlich': 'tablet',
    }

    def select(self):
        assignments = (
            self.db.query(
                Assignment.id.label('#'),
                Assignment.day.label('Datum'),
                Assignment.purpose.label('Zweck'),
                to_string(Member.lastname).concat(', ').concat(
                    to_string(Member.firstname)).label('Verantwortlich'))
            .select_from(Assignment)
            .join(Member)
            .filter(Assignment.accounting_year == get_selected_year()))
        return assignments


@view_config(route_name='assignments',
             renderer='json',
             permission='view')
class AssignmentListView(sw.allotmentclub.browser.base.TableView):
    """ Liste aller Arbeitseinsätze, filterbar nach Jahr."""

    query_class = Query
    default_order_by = 'day'
    year_selection = True
    start_year = 2015
    available_actions = [
        dict(url='assignment_add', btn_class='btn-success',
             icon='fa fa-plus', title='Neu'),
        dict(url='assignment_edit', btn_class='btn-success',
             icon='fa fa-pencil', title='Bearbeiten'),
        dict(url='assignment_delete', btn_class='btn-danger',
             icon='glyphicon glyphicon-trash', title='Löschen',
             callback='sw.allotmentclub.assignments_list_view.render()'),
        dict(url='assignment_list_attendees', btn_class='btn-success',
             icon='fa fa-list', title='Teilnehmer')]


@view_config(route_name='assignment_edit', renderer='json',
             permission='view')
class AssignmentEditView(sw.allotmentclub.browser.base.EditJSFormView):

    title = 'Arbeitseinsatz bearbeiten'

    @property
    def load_options(self):
        return {
            'day': {
                'label': 'Tag',
                'css_class': 'datetimepicker'
            },
            'purpose': {
                'label': 'Zweck',
            },
            'responsible_id': {
                'label': 'Verantwortlich',
                'source': self.member_source,
                'css_class': 'chosen',
                'required': True
            }
        }

    @property
    def load_data(self):
        fields = [
            ('day', date_time(self.context.day)),
            ('purpose', self.context.purpose),
            ('responsible_id', self.context.responsible_id),
        ]
        return collections.OrderedDict(fields)


@view_config(route_name='assignment_add', renderer='json',
             permission='view')
class AssignmentAddView(AssignmentEditView):

    title = 'Arbeitseinsatz anlegen'

    def __init__(self, context, request):
        context = Assignment.create(
            accounting_year=get_selected_year())
        context.commit()
        super(AssignmentAddView, self).__init__(context, request)
        log_with_user(user_data_log.info, self.request.user,
                      'Arbeitseinsatz %s hinzugefügt.', self.context.id)

    @property
    def route_name(self):
        return 'assignment_edit'


@view_config(route_name='assignment_delete', renderer='json',
             permission='view')
class AssignmentDeleteView(sw.allotmentclub.browser.base.DeleteView):

    model = Assignment


class QueryAttendees(sw.allotmentclub.browser.base.Query):

    def select(self):
        return (
            self.db.query(
                AssignmentAttendee.id.label('#'),
                to_string(Member.lastname).concat(', ').concat(
                    to_string(Member.firstname)).label('Teilnehmer'),
                AssignmentAttendee.hours.label('Stunden'))
            .select_from(AssignmentAttendee)
            .join(Member)
            .filter(AssignmentAttendee.assignment == self.context))


@view_config(route_name='assignment_list_attendees', renderer='json',
             permission='view')
class AssignmentListAttendeesView(sw.allotmentclub.browser.base.TableView):
    """ Liste aller Teilnehmer zu einem Arbeitseinsatz."""

    query_class = QueryAttendees
    default_order_by = 'lastname'
    available_actions = [
        dict(url='assignment_attendees_add', btn_class='btn-success',
             icon='fa fa-plus', title='Neu'),
        dict(url='assignment_attendees_edit', btn_class='btn-success',
             icon='fa fa-pencil', title='Bearbeiten'),
        dict(url='assignment_attendees_delete', btn_class='btn-danger',
             icon='glyphicon glyphicon-trash', title='Löschen',
             callback='sw.allotmentclub.assignment_attendees_view.render()')]


@view_config(route_name='assignment_attendees_edit', renderer='json',
             permission='view')
class AssignmentAttendeesEditView(
        sw.allotmentclub.browser.base.EditJSFormView):

    title = 'Stunden für Teilnehmer erfassen'

    @property
    def load_options(self):
        return {
            'member_id': {
                'label': 'Mitglied',
                'source': self.member_source,
                'css_class': 'chosen',
                'required': True
            },
            'hours': {
                'label': 'geleistete Stunden'
            }
        }

    @property
    def load_data(self):
        fields = [
            ('member_id', self.context.member_id),
            ('hours', self.context.hours),
        ]
        return collections.OrderedDict(fields)

    def save(self, key, value):
        if value and key == 'hours':
            value = value.replace(',', '.')
        return super(AssignmentAttendeesEditView, self).save(key, value)

    def get_route(self, item, name):
        route = super(AssignmentAttendeesEditView, self).get_route(item, name)
        return route.replace(
            '{assignment_id}', str(self.context.assignment.id))


@view_config(route_name='assignment_attendees_add', renderer='json',
             permission='view')
class AssignmentAttendeesAddView(AssignmentAttendeesEditView):

    model = AssignmentAttendee

    def __init__(self, context, request):
        context = AssignmentAttendee.create(assignment=context)
        super(AssignmentAttendeesAddView, self).__init__(context, request)

    @property
    def route_name(self):
        return 'assignment_attendees_edit'


@view_config(route_name='assignment_attendees_delete', renderer='json',
             permission='view')
class AssignmentAttendeesDeleteView(sw.allotmentclub.browser.base.DeleteView):

    model = AssignmentAttendee

    def log(self):
        if self.deleted is not None:
            deleted = self.context.member
            assignment = self.context.assignment.id
            log_with_user(user_data_log.info,
                          self.request.user,
                          '%s %s von Arbeitseinsatz %s gestrichen.',
                          deleted.firstname, deleted.lastname, assignment)


class MemberAssignmentQuery(sw.allotmentclub.browser.base.Query):

    formatters = {
        'Stunden': lambda mid, request: Member.get(mid).assignment_hours,
    }
    data_class = {
        'Mitglied': 'expand'
    }
    data_hide = {
        'Stunden': 'phone',
    }

    def select(self):
        return (
            self.db.query(
                Member.id.label('#'),
                (to_string(Member.lastname).concat(', ').concat(
                 to_string(Member.firstname).concat(' (')
                 .concat(func.coalesce(string_agg(Allotment.number), 'n/a'))
                 .concat(')'))).label('Mitglied'),
                Member.id.label('Stunden'),
            )
            .select_from(Allotment)
            .join(Member, Allotment.member_id == Member.id)
            .group_by(Member.id)
        )


@view_config(route_name='member_assignments', renderer='json',
             permission='view')
class MemberAssignmentView(sw.allotmentclub.browser.base.TableView):

    query_class = MemberAssignmentQuery
    year_selection = True
    start_year = 2015

    available_actions = [
        dict(url='member_assignments_detail', btn_class='btn-success',
             icon='fa fa-list', title='Details'),
        dict(url='member_assignments_bill', btn_class='btn-danger',
             icon='fa fa-bank', title='Abrechnen')]


class MemberAssignmentDetailQuery(sw.allotmentclub.browser.base.Query):

    formatters = {
        'Datum': date_time
    }

    data_class = {
        'Einsatz': 'expand'
    }

    def select(self):
        return (
            self.db.query(
                AssignmentAttendee.id.label('#'),
                Assignment.purpose.label('Einsatz'),
                Assignment.day.label('Datum'),
                AssignmentAttendee.hours.label('Stunden'))
            .select_from(AssignmentAttendee)
            .join(Assignment)
            .filter(AssignmentAttendee.member == self.context)
            .filter(Assignment.accounting_year == get_selected_year()))


@view_config(route_name='member_assignments_detail', renderer='json',
             permission='view')
class MemberAssignmentsDetailListView(sw.allotmentclub.browser.base.TableView):

    query_class = MemberAssignmentDetailQuery
    year_selection = True


@view_config(route_name='member_assignments_bill', renderer='json',
             permission='view')
class MemberAssignmentsBillView(sw.allotmentclub.browser.base.View):

    def update(self):
        for member in self.active_members.all():
            member.bill_assignment_hours()
        self.result = {'status': 'success',
                       'message': 'Arbeitsstunden erfolgreich übertragen'}



"""re #16: Add organization model.

Revision ID: f86ea5808406
Revises: 1dfd521b7ca0
Create Date: 2016-07-26 13:19:46.031735

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = 'f86ea5808406'
down_revision = '1dfd521b7ca0'

TABLES = ('abwasser', 'accessauthority', 'allotment', 'assignment',
          'assignmentattendee', 'attachment', 'bankingaccount', 'booking',
          'bookingkind', 'bulletin', 'depot', 'electricmeter', 'energyprice',
          'energyvalue', 'externalrecipient', 'grundsteuerb', 'key', 'keylist',
          'keylistattachment', 'log', 'member', 'message', 'parcel',
          'protocol', 'protocolattachment', 'protocolcommitment',
          'protocoldetail', 'salehistory', 'sepasammler', 'sepasammlerentry',
          'user')


def upgrade():
    op.create_table(
        'organization',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('title', sa.String(), nullable=True),
        sa.PrimaryKeyConstraint('id')
    )
    op.execute("""INSERT into organization (id, title)
                  VALUES (1, 'Leuna-Bungalowgemeinschaft Roter See')""")
    for table in TABLES:
        op.add_column(
            table,
            sa.Column('organization_id', sa.Integer(), nullable=True))
        upd_table = table
        if upd_table == 'user':
            upd_table = 'public.user'
        op.execute("UPDATE {} SET organization_id = 1".format(upd_table))
        op.alter_column(table, "organization_id", nullable=False)


def downgrade():
    for table in TABLES:
        op.drop_column(table, 'organization_id')
    op.drop_table('organization')



"""Add filter for ignoring bookings in reporting.

Revision ID: 1fb65d65969
Revises: 433297b9762
Create Date: 2015-11-17 15:04:58.595746

"""

# revision identifiers, used by Alembic.
from alembic import op
import sqlalchemy as sa

revision = '1fb65d65969'
down_revision = '433297b9762'


def upgrade():
    op.add_column(
        'booking',
        sa.Column('ignore_in_reporting',
                  sa.Boolean(),
                  server_default='False',
                  nullable=False))
    op.add_column(
        'sepasammler',
        sa.Column('ignore_in_reporting',
                  sa.Boolean(),
                  server_default='False',
                  nullable=False))
    op.execute("""UPDATE booking SET ignore_in_reporting = True
                  WHERE id in (102, 586, 516, 598, 591, 963, 431, 1001, 998,
                  999, 1000, 1027);""")
    op.execute("""UPDATE sepasammler SET ignore_in_reporting = True
                  WHERE id in (40, 266, 322, 320, 354, 324);""")


def downgrade():
    op.drop_column('booking', 'ignore_in_reporting')
    op.drop_column('sepasammler', 'ignore_in_reporting')



"""re #18: Add attachment to keylist.

Revision ID: d1ccee162c46
Revises: a08b46e0172e
Create Date: 2016-07-25 13:30:03.833379

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = 'd1ccee162c46'
down_revision = 'a08b46e0172e'


def upgrade():
    op.create_table(
        'keylistattachment',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('keylist_id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=100), nullable=True),
        sa.Column('mimetype', sa.String(length=30), nullable=True),
        sa.Column('size', sa.String(length=20), nullable=True),
        sa.Column('data', sa.LargeBinary(length=10485760), nullable=True),
        sa.ForeignKeyConstraint(
            ['keylist_id'], [u'keylist.id'], ondelete=u'cascade'),
        sa.PrimaryKeyConstraint('id')
    )


def downgrade():
    op.drop_table('keylistattachment')



"""Add formbrief depot.

Revision ID: 2535014c8ab8
Revises: 5154afa1d7d6
Create Date: 2015-01-03 14:38:20.323173

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '2535014c8ab8'
down_revision = '5154afa1d7d6'


def upgrade():
    op.create_table(
        'formletter',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column(
            'kind',
            sa.Enum(u'Standard', u'Abrechnung Energie',
                    u'Abschl\xe4ge Energie',
                    u'Einladung Mitgliederversammlung',
                    name='form_letter_kind'),
            nullable=False),
        sa.Column('status',
                  sa.Enum(u'In Bearbeitung', u'Fehler', u'Fertig gestellt',
                          name='form_letter_status'),
                  nullable=False),
        sa.Column('mimetype', sa.String(length=30), nullable=True),
        sa.Column('size', sa.String(length=20), nullable=True),
        sa.Column('data', sa.LargeBinary(length=10485760), nullable=True),
        sa.PrimaryKeyConstraint('id')
    )


def downgrade():
    op.drop_table('formletter')



"""Add reference to booking to SEPASammler.

Revision ID: 4d63c477b58f
Revises: 388a201feabf
Create Date: 2015-04-07 10:44:29.267471

"""

# revision identifiers, used by Alembic.
from alembic import op
import sqlalchemy as sa

revision = '4d63c477b58f'
down_revision = '388a201feabf'


def upgrade():
    op.add_column(
        'sepasammler',
        sa.Column('booking_day', sa.Date(), nullable=True))
    op.add_column(
        'sepasammler',
        sa.Column('accounting_year', sa.Integer(), nullable=True))
    op.execute("""DELETE FROM booking WHERE purpose like 'Lastschrift%';""")
    op.execute("""UPDATE sepasammler SET
                  accounting_year = 2015, booking_day = '2015-03-31';""")


def downgrade():
    op.drop_column('sepasammler', 'splitted_from_id')



"""Add deleted flag to external recipients

Revision ID: 7e15fdd9747a
Revises: 2952c5487114
Create Date: 2021-01-11 07:01:46.325174

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '7e15fdd9747a'
down_revision = '2952c5487114'


def upgrade():
    op.add_column(
        'externalrecipient',
        sa.Column('deleted', sa.Boolean(), nullable=True)
    )


def downgrade():
    op.drop_column('externalrecipient', 'deleted')



def test_member_active_passive(database):
    from sw.allotmentclub import Member, Allotment

    member = Member.create()

    assert member.active is False

    Allotment.create(member=member)

    assert member.active is True



"""Update formbrief

Revision ID: 179b760ddcda
Revises: 2535014c8ab8
Create Date: 2015-01-03 14:47:15.191085

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '179b760ddcda'
down_revision = '2535014c8ab8'


def upgrade():
    op.add_column('formletter',
                  sa.Column('date', sa.DateTime(), nullable=False))
    op.add_column('formletter',
                  sa.Column('user_id', sa.Integer(), nullable=False))


def downgrade():
    op.drop_column('formletter', 'user_id')
    op.drop_column('formletter', 'date')



"""add account for am.

Revision ID: 769e10087d9e
Revises: fd9f9588878d
Create Date: 2017-06-19 14:02:23.740567

"""
from alembic import op
import pkg_resources
import base64

# revision identifiers, used by Alembic.
revision = '769e10087d9e'
down_revision = 'fd9f9588878d'


def upgrade():
    signature = pkg_resources.resource_stream(
        'sw.allotmentclub.signatures', 'am.png').read()
    signature = 'data:application/png;base64,{}'.format(
        base64.b64encode(signature).decode('utf-8'))
    op.execute("""
INSERT into public.user
    (username, password, unrestricted_access, vorname, nachname, anrede,
     handynummer, ort, position, signature, email, organization_id)
VALUES
    ('am', '{pw}', false, 'Andreas', 'Mielke', 'Herr', '0160 1546148',
     'Naumburg', 'Schriftführer', '{signature}', 'amielke@primacom.net', 1)
""".format(pw='$2b$12$mi1JV2RA/hvmkQVVv2Uube4ODAX44LOsmMXvkwLUVlIzKS18P0eve',
           signature=signature))

    op.execute("""
INSERT into public.user
    (username, password, unrestricted_access, vorname, nachname, anrede,
     handynummer, ort, position, signature, email, organization_id)
VALUES
    ('cs', '{pw}', false, 'Constanze', 'Seyfert', 'Frau', '',
     '', 'Revisionskommission', '', '', 1)
""".format(pw='$2b$12$i8SjfxgMnw464DAyFJEipeXswIvtC.OcrSGYYZ/PBAC97NvJus0ou'))

    op.execute("""
UPDATE public.user SET is_locked = 'Zurückgetreten' WHERE username = 'ks'""")
    op.execute("""
UPDATE public.user SET position = 'Datenbank-Beauftragter'
WHERE username = 'sw'""")
    op.execute("""
UPDATE public.user SET position = 'Vorsitzender'
WHERE username = 'hs'""")
    op.execute("""
UPDATE public.user SET position = 'Stellvertretende Vorsitzende'
WHERE username = 'ar'""")


def downgrade():
    op.execute("DELETE FROM public.user WHERE username = 'am';")
    op.execute("DELETE FROM public.user WHERE username = 'cs';")



"""Add inbound flag for messages.

Revision ID: b01e97903b78
Revises: 78f2790c1501
Create Date: 2017-05-12 09:11:30.803425

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = 'b01e97903b78'
down_revision = '78f2790c1501'


def upgrade():
    op.add_column('message',
                  sa.Column('inbound', sa.Boolean(), nullable=False,
                            server_default=sa.false()))


def downgrade():
    op.drop_column('message', 'inbound')



# encoding=utf8
from __future__ import unicode_literals
from ..log import user_data_log, log_with_user
from .base import to_string
from pyramid.view import view_config
from sw.allotmentclub import ExternalRecipient
from sqlalchemy import or_, and_
import collections
import sw.allotmentclub.browser.base


def filter_external_recipients(query):
    return (
        query.filter(
            or_(
                and_(
                    ExternalRecipient.city != '',
                    ExternalRecipient.city.isnot(None),
                ),
                and_(
                    ExternalRecipient.email != '',
                    ExternalRecipient.email.isnot(None),
                )
            )
        )
        .filter(
            and_(
                ExternalRecipient.lastname != '',
                ExternalRecipient.lastname.isnot(None),
            )
        )
        .filter(ExternalRecipient.deleted.isnot(True))
    )


class Query(sw.allotmentclub.browser.base.Query):

    formatters = {}
    data_class = {
        'Nachname': 'expand'
    }
    data_hide = {
        'Vorname': 'phone,tablet',
        'Adresse': 'phone,tablet',
        'E-Mail': 'phone',
    }

    def select(self):
        return filter_external_recipients(
            self.db.query(
                ExternalRecipient.id.label('#'),
                ExternalRecipient.organization.label('Firma'),
                ExternalRecipient.lastname.label('Nachname'),
                ExternalRecipient.firstname.label('Vorname'),
                (
                    to_string(ExternalRecipient.street).concat('<br />')
                    .concat(to_string(ExternalRecipient.zip)).concat(' ')
                    .concat(to_string(ExternalRecipient.city))
                ).label('Adresse'),
                ExternalRecipient.email.label('E-Mail'),
            )
            .select_from(ExternalRecipient)
        )


@view_config(route_name='externals', renderer='json',
             permission='view')
class ExternalRecipientListView(sw.allotmentclub.browser.base.TableView):

    query_class = Query
    default_order_by = 'lastname'
    available_actions = [
        dict(url='external_add', btn_class='btn-success', icon='fa fa-plus',
             title='Neu'),
        dict(url='external_edit', btn_class='btn-success', icon='fa fa-pencil',
             title='Bearbeiten'),
        dict(url='external_delete', btn_class='btn-danger',
             icon='glyphicon glyphicon-trash', title='Löschen')]


@view_config(route_name='external_edit', renderer='json',
             permission='view')
class ExternalRecipientEditView(sw.allotmentclub.browser.base.EditJSFormView):

    title = 'Externen Empfänger bearbeiten'

    @property
    def load_options(self):
        return {
            'title': {
                'label': 'Titel',
                'required': True,
                'source': [
                    {'title': '', 'token': ''},
                    {'title': 'Dr.', 'token': 'Dr.'},
                    {'title': 'Dr. med.', 'token': 'Dr. med.'},
                    {'title': 'Prof. Dr.', 'token': 'Prof. Dr.'}
                ]},
            'appellation': {
                'required': True,
                'label': 'Anrede',
                'source': [
                    {'title': '', 'token': ''},
                    {'title': 'Herr', 'token': 'Herr'},
                    {'title': 'Frau', 'token': 'Frau'}
                ]},
            'organization': {'label': 'Firma'},
            'firstname': {'label': 'Vorname', 'required': True},
            'lastname': {'label': 'Nachname', 'required': True},
            'street': {'label': 'Straße'},
            'zip': {'label': 'PLZ'},
            'city': {'label': 'Stadt'},
            'country': {'label': 'Land'},
            'email': {'label': 'E-Mail'},
            'phone': {'label': 'Telefon'},
        }

    @property
    def load_data(self):
        fields = [
            ('appellation', self.context.appellation),
            ('title', self.context.title),
            ('organization', self.context.organization),
            ('firstname', self.context.firstname),
            ('lastname', self.context.lastname),
            ('street', self.context.street),
            ('zip', self.context.zip),
            ('city', self.context.city),
            ('country', self.context.country),
            ('email', self.context.email),
            ('phone', self.context.phone),

        ]
        return collections.OrderedDict(fields)


@view_config(route_name='external_add', renderer='json',
             permission='view')
class ExternalRecipientAddView(ExternalRecipientEditView):

    title = 'Externen Empfänger anlegen'

    def __init__(self, context, request):
        context = ExternalRecipient.create()
        context.commit()
        super(ExternalRecipientAddView, self).__init__(context, request)
        log_with_user(
            user_data_log.info, self.request.user,
            'Externen Empfänger %s hinzugefügt.', self.context.id)

    @property
    def route_name(self):
        return 'external_edit'


@view_config(route_name='external_delete', renderer='json',
             permission='view')
class ExternalRecipientDeleteView(sw.allotmentclub.browser.base.DeleteView):

    model = ExternalRecipient
    mark_deleted = True



"""Add signature for ar

Revision ID: 5b929a47d6ae
Revises: 4841df08d22b
Create Date: 2016-04-18 10:28:47.331715

"""

from alembic import op
import base64
import pkg_resources

# revision identifiers, used by Alembic.
revision = '5b929a47d6ae'
down_revision = '4841df08d22b'


def upgrade():
    signature = pkg_resources.resource_stream(
        'sw.allotmentclub.signatures', 'ar.png').read()
    signature = 'data:application/png;base64,{}'.format(
        base64.b64encode(signature))
    op.execute(u"""UPDATE public.user SET
                  signature = '{signature}'
                  WHERE username = 'ar';""".format(signature=signature))


def downgrade():
    op.execute(u"""UPDATE public.user SET
                  signature = ''
                  WHERE username = 'ar';""")



"""Add printed timestamp to mail.

Revision ID: 433297b9762
Revises: 38a169e80b6e
Create Date: 2015-11-17 07:13:52.623564

"""

# revision identifiers, used by Alembic.
from alembic import op
import sqlalchemy as sa

revision = '433297b9762'
down_revision = '38a169e80b6e'


def upgrade():
    op.add_column('message',
                  sa.Column('printed', sa.DateTime(), nullable=True))


def downgrade():
    op.drop_column('message', 'printed')



"""Add new groups

Revision ID: 4ac1a4b79213
Revises: 1850226c55dc
Create Date: 2014-12-16 20:58:41.652962

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '4ac1a4b79213'
down_revision = '1850226c55dc'


def upgrade():
    op.add_column(
        'user', sa.Column('can_modify', sa.Boolean(), server_default='false',
                          nullable=False))
    op.add_column(
        'user', sa.Column('read_only', sa.Boolean(), server_default='false',
                          nullable=False))
    op.drop_column('user', 'may_grant')


def downgrade():
    op.add_column(
        'user', sa.Column('may_grant', sa.BOOLEAN(), server_default='false',
                          nullable=False))
    op.drop_column('user', 'read_only')
    op.drop_column('user', 'can_modify')



"""Add subject to formletters.

Revision ID: 1c44d30192ea
Revises: 3c0b1fc7eeed
Create Date: 2015-01-26 15:37:25.999671

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '1c44d30192ea'
down_revision = '3c0b1fc7eeed'


def upgrade():
    op.add_column('formletter',
                  sa.Column('subject', sa.String(), nullable=True))


def downgrade():
    op.drop_column('formletter', 'subject')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import transaction
from ...tests.test_account import import_bookings


def setUp():
    import_bookings()
    transaction.commit()


def test_BookingListView_1(browser, json_fixture):
    """It displays list of bookings."""
    url = json_fixture.url()
    setUp()
    browser.login()
    browser.open('http://localhost{}'.format(url))
    json_fixture.assertEqual(browser.json, 'data')


def test_SplitBookingView_1(browser, json_fixture):
    """It displays a form with one input field for the split value."""
    url = json_fixture.url('/booking/1/split')
    setUp()
    browser.login()
    browser.open('http://localhost{}'.format(url))
    json_fixture.assertEqual(browser.json, 'data')


def test_SplitBookingView_2(browser, json_fixture):
    """It splits booking on form submit."""
    from sw.allotmentclub import Booking
    setUp()
    assert len(Booking.query().all()) == 30
    browser.login()
    json_fixture.ajax(browser)
    assert 'success' == browser.json['status']
    assert Booking.get(1).is_splitted is True
    assert len(Booking.query().all()) == 32


def test_SplitBookingView_3(browser, json_fixture):
    """It shows an error message if split failed."""
    from sw.allotmentclub import Booking
    setUp()
    assert len(Booking.query().all()) == 30
    browser.login()
    json_fixture.ajax(browser)
    assert 'error' == browser.json['status']
    expmsg = 'Neuer Wert muss gr\xf6\xdfer oder kleiner 0 sein.'
    assert expmsg == browser.json['msg']
    assert len(Booking.query().all()) == 30
    assert Booking.get(1).is_splitted is False


def setUpSEPASammlerUpdate(kind_shorttitle):
    from sw.allotmentclub import (
        SEPASammler, Booking, BookingKind, BankingAccount, Member)
    kind = BookingKind.find_or_create(title='', shorttitle=kind_shorttitle)
    sammler = SEPASammler.create(
        booking_day='2016-06-30', accounting_year=2016, kind=kind)
    for value in [-2500, -821300]:
        Booking.find_or_create(
            value=value, banking_account=BankingAccount.find_or_create(),
            accounting_year=2016, kind=kind,
            member=Member.create(direct_debit=True))
    setUp()
    return sammler


def test_SEPASammlerUpdateView_1(browser):
    """Testing SEPASammler Energieabschlag 1."""
    from sw.allotmentclub import SEPASammlerEntry
    sammler = setUpSEPASammlerUpdate('ENA1')
    assert len(SEPASammlerEntry.query().all()) == 0
    browser.login()
    browser.open(
        'http://localhost/accounts/sepa_sammler/{}/update'.format(sammler.id))
    assert len(SEPASammlerEntry.query().all()) == 2
    SEPASammlerEntry.query().filter(SEPASammlerEntry.value == 821300).one()


def test_SEPASammlerUpdateView_2(browser):
    """Testing SEPASammler Energieabschlag 2."""
    from sw.allotmentclub import SEPASammlerEntry
    sammler = setUpSEPASammlerUpdate('ENA1')
    assert len(SEPASammlerEntry.query().all()) == 0
    browser.login()
    browser.open(
        'http://localhost/accounts/sepa_sammler/{}/update'.format(sammler.id))
    assert len(SEPASammlerEntry.query().all()) == 2
    SEPASammlerEntry.query().filter(SEPASammlerEntry.value == 821300).one()


def test_SEPASammlerExportView_1(browser):
    """Testing SEPASammler Energieabschlag 1 XML export."""
    from sw.allotmentclub import SEPASammler, SEPASammlerEntry, BookingKind
    from sw.allotmentclub import Member
    import datetime
    import lxml.etree
    kind = BookingKind.find_or_create(
        title='Energieabschlag I', shorttitle='ENA1')
    sammler = SEPASammler.create(
        booking_day='2018-03-31', accounting_year=2018, kind=kind)
    for iban, value in (('DE12500105170648489890', 2540500),
                        ('EE342200221034126658', 8213400)):
        SEPASammlerEntry.find_or_create(
            sepasammler=sammler, value=value,
            member=Member.create(
                lastname='Müller', direct_debit=True, iban=iban,
                bic='NOLADE21HAL',
                direct_debit_date=datetime.date(2017, 1, 1)))
    setUp()
    browser.login()
    browser.open(
        'http://localhost/accounts/sepa_sammler/{}/export'.format(sammler.id))
    doc = lxml.etree.fromstring(browser.contents.encode('utf-8'))
    assert '1075.39' == doc.find('.//CtrlSum', namespaces=doc.nsmap).text
    assert 'Mueller, ' == doc.findall('.//Nm', namespaces=doc.nsmap)[-1].text
    assert 'MUELLER' == doc.find('.//MndtId', namespaces=doc.nsmap).text


def test_SEPASammlerExportView_2(browser):
    """Testing SEPASammler Energieabrechnung Sammelüberweisung XML export."""
    from sw.allotmentclub import SEPASammler, SEPASammlerEntry, BookingKind
    from sw.allotmentclub import Member
    import datetime
    import lxml.etree
    kind = BookingKind.find_or_create(
        title='Energieabrechnung', shorttitle='ENAB')
    sammler = SEPASammler.create(
        booking_day='2018-03-31', accounting_year=2018, kind=kind,
        is_ueberweisung=True)
    for iban, value in (('DE12500105170648489890', 2540500),
                        ('EE342200221034126658', 8213400)):
        SEPASammlerEntry.find_or_create(
            sepasammler=sammler, value=value,
            member=Member.create(
                lastname='Müller', direct_debit=True, iban=iban,
                bic='NOLADE21HAL',
                direct_debit_date=datetime.date(2017, 1, 1)))
    setUp()
    browser.login()
    browser.open(
        'http://localhost/accounts/sepa_sammler/{}/export'.format(sammler.id))
    doc = lxml.etree.fromstring(browser.contents.encode('utf-8'))
    assert '1075.39' == doc.find('.//CtrlSum', namespaces=doc.nsmap).text
    assert '2' == doc.find('.//NbOfTxs', namespaces=doc.nsmap).text
    assert ['254.05', '821.34'] == [
        d.text for d in doc.findall('.//InstdAmt', namespaces=doc.nsmap)
    ]


def test_BankingAccountListReportView_1(browser):
    from sw.allotmentclub import Booking, BookingKind, Member
    from ...conftest import assertFileEqual
    import transaction
    kind = BookingKind.find_or_create(
        title='Energieabschlag I', shorttitle='ENA1')
    BookingKind.find_or_create(
        title='Energieabschlag II', shorttitle='ENA2')
    member = Member.find_or_create(lastname='Wehrmann', firstname='Sebastian')
    setUp()
    for b in Booking.query():
        b.kind = kind
        b.member = member
    transaction.commit()

    browser.login()
    browser.open('http://localhost/accounts/report.pdf?for_year=2015')
    assertFileEqual(browser.contents, 'test_account_report_1.pdf')



"""Add booking kinds.

Revision ID: 568f7817789f
Revises: 1039814b69e3
Create Date: 2015-03-11 09:32:08.578225

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '568f7817789f'
down_revision = '1039814b69e3'


def upgrade():
    op.create_table(
        'bookingkind',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('title', sa.String(length=20), nullable=False),
        sa.Column('year', sa.Integer(), nullable=True),
        sa.Column('value_per_member', sa.Integer(), nullable=True),
        sa.PrimaryKeyConstraint('id')
    )
    op.add_column(
        u'booking', sa.Column('kind_id', sa.Integer(), nullable=True))
    op.execute("""INSERT into bookingkind (title, year, value_per_member)
                  VALUES ('Mitgliedsbeitrag', 2015, 600000);""")
    op.execute("""INSERT into bookingkind (title, year, value_per_member)
                  VALUES ('Fehl.Arb.Std', 2015, 50);""")
    op.execute("""INSERT into bookingkind (title, year)
                  VALUES ('Energieabschlag I', 2015);""")
    op.execute("""INSERT into bookingkind (title, year)
                  VALUES ('Energieabschlag II', 2015);""")
    op.execute("""INSERT into bookingkind (title, year)
                  VALUES ('Energieabrechnung', 2015);""")


def downgrade():
    op.drop_column(u'booking', 'kind_id')
    op.drop_table('bookingkind')



"""Add charge and advance pay from electricity facroty.

Revision ID: 5154afa1d7d6
Revises: 63317db97d4
Create Date: 2014-12-25 08:59:45.936941

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '5154afa1d7d6'
down_revision = '63317db97d4'


def upgrade():
    op.add_column(
        'energyprice',
        sa.Column('advance_pay', sa.Integer(), nullable=True))
    op.add_column(
        'energyprice',
        sa.Column('charge', sa.Integer(), nullable=True))
    op.execute("UPDATE energyprice SET charge = 122910900 WHERE year = 2014;")
    op.execute(
        "INSERT into energyprice (year, advance_pay) VALUES (2015, 10500000);")


def downgrade():
    op.drop_column('energyprice', 'charge')
    op.drop_column('energyprice', 'advance_pay')



from pyramid.security import Allow, Authenticated
from sw.allotmentclub import User, AccessAuthority
import pyramid.httpexceptions
import pyramid.security
import pyramid.threadlocal


def authorize(request=None, context=None):
    if request is None:
        request = pyramid.threadlocal.get_current_request()
    route_name = request.matched_route.name
    user = request.user
    if context is not None:
        if user.organization_id != context.organization_id:
            return []
    allowed = (AccessAuthority.query()
               .filter(AccessAuthority.viewname == route_name)
               .filter(AccessAuthority.user == user).all())
    if user.unrestricted_access or allowed:
        return [(Allow, Authenticated, 'view')]
    return []


class DefaultContext(object):

    @property
    def __acl__(self):
        return authorize(self.request)

    def __init__(self, request):
        self.request = request


def get_default_context(request):
    return DefaultContext(request)


def login(login, password):
    """Try to verify the user.

    Returns (User, authenticated?)

    User will be None if password is missing or user does not exist,
    otherwise the object will be retrieved from the database.

    authenticated? will be True iff check_password succeeds
    and False otherwise.
    """
    user = None
    if login:
        user = User.by_username(login)
    if user and password:
        return (user, user.check_password(password))

    return (None, False)


def get_user(request):
    user_id = request.authenticated_userid
    if user_id:
        return User.get(user_id)



"""Add keylist model.

Revision ID: 4b98c95b9a56
Revises: 1842f7373dcd
Create Date: 2016-07-06 20:22:33.558515

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '4b98c95b9a56'
down_revision = '1842f7373dcd'


def upgrade():
    op.create_table(
        'keylist',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('subject', sa.String(), nullable=True),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_table(
        'key',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('keylist_id', sa.Integer(), nullable=False),
        sa.Column('serial', sa.String(), nullable=True),
        sa.Column('member_id', sa.Integer(), nullable=True),
        sa.Column('rent', sa.DateTime(), nullable=True),
        sa.Column('note', sa.String(), nullable=True),
        sa.Column('lost', sa.Boolean(), nullable=True, server_default='False'),
        sa.ForeignKeyConstraint(['keylist_id'], [u'keylist.id'], ),
        sa.ForeignKeyConstraint(['member_id'], [u'member.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    op.drop_constraint(u'key_keylist_id_fkey', 'key', type_='foreignkey')
    op.create_foreign_key(None, 'key', 'keylist', ['keylist_id'], ['id'],
                          ondelete=u'cascade')


def downgrade():
    op.drop_table('key')
    op.drop_table('keylist')



"""Add depot

Revision ID: 46276fff5bc2
Revises: 3ed04edde04e
Create Date: 2014-11-24 15:45:42.950371

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '46276fff5bc2'
down_revision = '3ed04edde04e'


def upgrade():
    op.create_table(
        'depot',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(length=100), nullable=True),
        sa.Column('size', sa.String(length=20), nullable=True),
        sa.Column('mimetype', sa.String(length=30), nullable=True),
        sa.Column('data', sa.LargeBinary(length=10485760), nullable=True),
        sa.Column('date', sa.DateTime(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(['user_id'], [u'user.id'], ),
        sa.PrimaryKeyConstraint('id')
    )


def downgrade():
    op.drop_table('depot')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from datetime import datetime
from sw.allotmentclub import Member
from io import BytesIO
import email.utils
import pkg_resources
import pybars
import pyramid.threadlocal
import pyramid_mailer
import pyramid_mailer.message
import re
import xhtml2pdf.pisa
import unicodedata

from reportlab import rl_config
rl_config.invariant = True


LOGO = pkg_resources.resource_stream(
    'sw.allotmentclub.browser', 'logo_letter.txt').read().decode('utf-8')

TEMPLATE_HEADER = """
<html lang="de">
<head>
<meta charset="utf-8"/>
<style>
    body { font-size: {{font_size}}; }
    #content { font-style: italic; }
    .txt-color-red { font-color: red; }
    {{{styles}}}
</style>
</head>

<body>
    <div id="header_content" style="font-size: 9pt;">
      <h1 style="font-family: Helvetica; color: #1f496d;
                 font-size: 16pt; margin: 0px; padding: 0px;">
        Leuna&mdash;Bungalowgemeinschaft Roter See e.V.</h1>
      Postfach 4023<br />
      06233 Leuna<br />
      vorstand@roter-see.de
    </div>
    <div id="logo">
      <img src="{{logo}}" />
    </div>
"""

TEMPLATE_FOOTER = """
    <table><tbody><tr><td>
      {{#if from_}}
      <p>Mit freundlichen Grüßen<br />Im Auftrag des Vorstandes</p>
      {{#if signature}}
        <img style="height: 25pt" src="{{signature}}" />
      {{else}}
        <br />
      {{/if}}
      <p style="font-style: italic;">
        {{{from_}}}
      </p>
      {{/if}}
    </td>
    <td style="vertical-align: bottom; text-align: right;">
      {{#if attachments}}
      <p style="font-size: {{font_size}}; padding-right: 10pt;">
        <span style="font-weight: bold;">Anlagen</span>
        {{#each attachments}}
          <br />{{title}}
        {{/each}}
      </p>
      {{/if}}
    </td></td></tbody></table>
    <div id="footer_content" style="font-size: 9pt">
      <table style="margin: 0px; padding: 0px;">
        <tr>
          <td style="width: 35%; text-align: left;">
          Amtsgericht Stendal VR 46284</td>
          <td style="width: 30%; text-align: center;"></td>
          <td style="width: 35%; text-align: right;">
          Vorsitzender: Hartmut Schlöffel</td>
        </tr>
      </table>
      <hr style="margin: 0px; padding: 0px;" />
      <table style="margin: 0px; padding: 0px; margin-top: 3pt;">
        <tr>
          <td style="width: 35%; text-align: left;">Saalesparkasse</td>
          <td style="width: 30%; text-align: center;">BIC: NOLADE21HAL</td>
          <td style="width: 35%; text-align: right;">
          IBAN: DE71 8005 3762 3440 0001 67</td>
        </tr>
      </table>
    </div>
</body>
</html>"""

PDF_TEMPLATE = TEMPLATE_HEADER + """
    {{#if address}}
    <p style="font-size: 7pt; text-decoration: underline;
              padding-bottom: 7pt;">
      Leuna&mdash;Bungalowgemeinschaft Roter See e.V.&nbsp;&nbsp;Postfach 4023
      &nbsp;&nbsp;06233 Leuna
    </p>
    <div style="font-size: 11pt; line-height: 11pt; padding-top: 13pt;">
      {{{address}}}
    </div>
    {{/if}}
    {{#if address}}
    <p style="text-align: right; width: 100%; padding-top: 30pt;
              margin-top: 0px;">
      {{city}}, den {{date}}</p>
    {{/if}}
    {{#if subject}}
    <h2 style="font-size: {{font_size}};">
      {{{subject}}}
      {{#if subsubject}}
      <br />
      <span style="font-weight: normal">{{{subsubject}}}</span><br />
      {{/if}}
    </h2>
    {{/if}}
    <div id="content">
      {{{content}}}
    </div>
""" + TEMPLATE_FOOTER

PDF_STYLES = """
    @page {
        size: a4 portrait;
        @frame header_frame {
            -pdf-frame-content: header_content;
            left: 50pt; width: 412pt; top: 50pt; height: 80pt;
        }
        @frame logo_frame {
            -pdf-frame-content: logo;
            left: 462pt; width: 00pt; top: 50px; height: 80pt;
        }
        @frame content_frame {
            left: 50pt; width: 512pt; top: 145pt; height: 600pt;
        }
        @frame footer_frame {
            -pdf-frame-content: footer_content;
            left: 50pt; width: 512pt; top: 752pt; height: 40pt;
        }
    }
"""

MAIL_TEMPLATE = TEMPLATE_HEADER + """
    <p style="text-align: right; width: 100%; padding-top: 30pt;
              margin-top: 0px;">
      {{city}}, den {{date}}</p>
    <div id="content">
      {{{content}}}
    </div>
""" + TEMPLATE_FOOTER

MAIL_STYLES = """
    #header_content { display: block; padding-top: 10pt; width: 80% }
    #logo { display: block; position: absolute; top: 8pt; right: 8pt; }
    #logo img { height: 50pt; }
    #footer_content { display: block; width: 100%; }
    #content { display: block; padding: 0pt 10pt 0pt 0pt; }
    table { padding: 0pt 10pt 20pt 0pt; font-size: 12pt; font-style: italic; }
    #footer_content table { width: 100%; font-size: 9pt; }
"""

compiler = pybars.Compiler()
pdf_template = compiler.compile(PDF_TEMPLATE)
mail_template = compiler.compile(MAIL_TEMPLATE)


def render_pdf(
        to, subject, content, user, date=None,
        subsubject=None, attachments=None, font_size='12pt', force_from=False):
    if date is None:
        date = datetime.now()
    if isinstance(to, Member):
        to = to.address
    if not to and not force_from:
        from_ = None
    else:
        from_ = '{} {} ({})'.format(user.vorname, user.nachname, user.position)
    data = dict(
        styles=PDF_STYLES,
        subject=subject,
        subsubject=subsubject,
        content=content,
        from_=from_,
        city=user.ort if user else None,
        signature=user.signature if user else None,
        date=date.strftime('%d.%m.%Y') if date else None,
        address=to if to else None, logo=LOGO,
        attachments=attachments, font_size=font_size)
    html = "".join(pdf_template(data))
    pdf = BytesIO()
    xhtml2pdf.pisa.CreatePDF(src=html, dest=pdf, encoding='utf-8')
    pdf.seek(0)
    return pdf


def send_mail(to, subject, content, user, date=None, attachments=[],
              font_size='12pt', self_cc=False):
    anschreiben = render_pdf(to, subject, content, user, date)
    if date is None:
        date = datetime.now()
    from_ = '{} {} ({})'.format(user.vorname, user.nachname, user.position)
    text = """
{}

Mit freundlichen Grüßen,
Im Auftrag des Vorstandes

{}

""".format(content, from_)

    text = text.replace('</p>', '\n')
    text = re.sub('<[^<]+?>', '', text)
    text = text.replace('  ', ' ')
    msg_tag = email.utils.make_msgid()
    msg = pyramid_mailer.message.Message(
        subject=subject,
        recipients=[to],
        cc=['vorstand@roter-see.de'] if self_cc else None,
        sender=('Vorstand Leuna-Bungalowgemeinschaft Roter See '
                '<vorstand@roter-see.de>'),
        body=text)
    msg.attach(pyramid_mailer.message.Attachment(
            'Anschreiben.pdf', 'application/pdf', anschreiben))
    for attachment in attachments:
        filename = attachment.filename
        try:
            # Prevent tuple exception in repoze.sendmail if filename contains
            # non-ascii characters
            filename = (unicodedata
                        .normalize('NFKD', filename)
                        .encode('ASCII', 'ignore')
                        .decode())
        except Exception:
            pass
        msg.attach(pyramid_mailer.message.Attachment(
            filename, attachment.mimetype, attachment.data))
    mailer = pyramid_mailer.get_mailer(
        pyramid.threadlocal.get_current_request())
    mailer.send(msg)
    return msg_tag



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .protocol import format_markdown
from .base import date_time
from ..log import user_data_log, log_with_user
from pyramid.view import view_config
from sw.allotmentclub import Bulletin, User
import collections
import sw.allotmentclub.browser.base
import sw.allotmentclub.browser.letter


class Query(sw.allotmentclub.browser.base.Query):

    formatters = {
        'Datum': date_time,
    }

    data_class = {
        'Betreff': 'expand'
    }
    data_hide = {
        'Datum': 'phone,tablet',
        'Von': 'phone,tablet',
    }

    def select(self):
        return (
            self.db.query(
                Bulletin.id.label('#'),
                Bulletin.subject.label('Betreff'),
                Bulletin.date.label('Datum'),
                User.username.label('Von'))
            .select_from(Bulletin)
            .join(User))


@view_config(route_name='bulletins', renderer='json',
             permission='view')
class BulletinListView(sw.allotmentclub.browser.base.TableView):
    """ Liste aller Aushänge."""

    query_class = Query
    default_order_by = 'date'
    available_actions = [
        dict(url='bulletin_add', btn_class='btn-success', icon='fa fa-plus',
             title='Neu'),
        dict(url='bulletin_edit', btn_class='btn-success', icon='fa fa-pencil',
             title='Bearbeiten'),
        dict(url='bulletin_delete', btn_class='btn-danger',
             icon='glyphicon glyphicon-trash', title='Löschen'),
        dict(url='bulletin_print', btn_class='btn-success',
             icon='fa fa-print', title='Herunterladen')]


@view_config(route_name='bulletin_edit', renderer='json',
             permission='view')
class BulletinEditView(sw.allotmentclub.browser.base.EditJSFormView):

    title = 'Aushang bearbeiten'

    @property
    def load_options(self):
        return {
            'subject': {
                'label': 'Betreff',
            },
            'content': {
                'label': 'Inhalt',
                'template': 'form_markdown',
            }
        }

    @property
    def load_data(self):
        fields = [
            ('subject', self.context.subject),
            ('content', self.context.content)]
        return collections.OrderedDict(fields)


@view_config(route_name='bulletin_add', renderer='json',
             permission='view')
class BulletinAddView(BulletinEditView):

    def __init__(self, context, request):
        context = Bulletin.create()
        context.commit()
        super(BulletinAddView, self).__init__(context, request)
        log_with_user(
            user_data_log.info, self.request.user,
            'Aushang %s hinzugefügt.', self.context.id)

    @property
    def route_name(self):
        return 'bulletin_edit'


@view_config(route_name='bulletin_delete', renderer='json',
             permission='view')
class BulletinDeleteView(sw.allotmentclub.browser.base.DeleteView):

    model = Bulletin


@view_config(route_name='bulletin_print', permission='view')
class BulletinPrintView(sw.allotmentclub.browser.base.PrintBaseView):

    filename = 'Aushang'

    def get_pdf(self):
        data = self.get_json(self.context)
        content = format_markdown('# %s\n\n%s' % (
            data['subject'], data['content']))
        content = '<div style="padding-top: 10pt">%s</div>' % content
        data['content'] = content
        return sw.allotmentclub.browser.letter.render_pdf(
            None, None, data['content'], self.context.user,
            self.context.date, font_size='16pt')



"""Remove dashboard

Revision ID: d0bfe38ddce2
Revises: 7e15fdd9747a
Create Date: 2021-02-10 13:32:51.989465

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "d0bfe38ddce2"
down_revision = "7e15fdd9747a"


def upgrade():
    op.drop_table("dashboarddata")


def downgrade():
    op.create_table(
        "dashboarddata",
        sa.Column(
            "organization_id",
            sa.INTEGER(),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "date",
            postgresql.TIMESTAMP(),
            server_default=sa.text("now()"),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "luefterstufe", sa.INTEGER(), autoincrement=False, nullable=False
        ),
        sa.Column(
            "luefter_percent",
            sa.INTEGER(),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "luefter_abluft_feuchte",
            sa.INTEGER(),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "luefter_aussenluft",
            sa.NUMERIC(precision=3, scale=1),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "luefter_zuluft",
            sa.NUMERIC(precision=3, scale=1),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "luefter_fortluft",
            sa.NUMERIC(precision=3, scale=1),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "luefter_abluft",
            sa.NUMERIC(precision=3, scale=1),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "rotersee_temp_out_battery",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "rotersee_rain_battery",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "rotersee_out_temp",
            sa.NUMERIC(precision=3, scale=1),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "rotersee_in_temp",
            sa.NUMERIC(precision=3, scale=1),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "rotersee_out_humi",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "rotersee_in_humi",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "rotersee_in_co2", sa.INTEGER(), autoincrement=False, nullable=True
        ),
        sa.Column(
            "rotersee_rain", sa.INTEGER(), autoincrement=False, nullable=True
        ),
        sa.Column(
            "rotersee_rain_1", sa.INTEGER(), autoincrement=False, nullable=True
        ),
        sa.Column(
            "rotersee_rain_24",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "rotersee_out_temp_trend",
            sa.VARCHAR(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_temp_out_battery",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_temp_in_battery",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_rain_battery",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_wind_battery",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_out_temp",
            sa.NUMERIC(precision=3, scale=1),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_in_temp",
            sa.NUMERIC(precision=3, scale=1),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_out_humi",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_in_humi",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_in_co2",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_rain",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_rain_1",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_rain_24",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_wind_strength",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_wind_angle",
            sa.INTEGER(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "wachtelberg_out_temp_trend",
            sa.VARCHAR(),
            autoincrement=False,
            nullable=True,
        ),
        sa.PrimaryKeyConstraint("date", name="dashboarddata_pkey"),
    )



import pyramid_mailer.interfaces
import pyramid_mailer.mailer
from sw.allotmentclub.log import app_log


class LogMailer(pyramid_mailer.mailer.Mailer):
    """
    Mailer which logs sending/creating of messages
    """

    def _log(self, action, message):
        app_log.info(u'{}: To: {} Subject: {}'.format(
            action, u', '.join(message.recipients), message.subject))

    def send(self, message):
        self._log(u'Mail sent', message)
        return super(LogMailer, self).send(message)

    def send_immediately(self, message, fail_silently=False):
        self._log(u'Mail sent immediately', message)
        return super(LogMailer, self).send_immediately(message, fail_silently)

    def send_to_queue(self, message):
        self._log(u'Mail added to queue', message)
        return super(LogMailer, self).send_to_queue(message)

    def send_sendmail(self, message):
        self._log(u'Mail sent via sendmail', message)
        return super(LogMailer, self).send_sendmail(message)

    def send_immediately_sendmail(self, message, fail_silently=False):
        self._log(u'Mail sent via sendmail immediately', message)
        return super(LogMailer, self).send_immediately_sendmail(
            message, fail_silently)


def includeme(config):
    settings = config.registry.settings
    prefix = settings.get('pyramid_mailer.prefix', 'mail.')
    mailer = LogMailer.from_settings(settings, prefix)
    config.registry.registerUtility(mailer, pyramid_mailer.interfaces.IMailer)



# encoding=utf-8
from pyramid.view import view_config
from sw.allotmentclub import Event, User, Protocol, Assignment, Member
from sw.allotmentclub import SEPASammler, BookingKind
from ..log import user_data_log, log_with_user
from .base import to_string, get_selected_year, date, date_time
from .letter import render_pdf
from sqlalchemy.dialects.postgresql import INTERVAL
from sqlalchemy.sql.functions import concat
import pybars
import sw.allotmentclub.browser.base
import pyramid.threadlocal
import sqlalchemy
import zope.component
import risclog.sqlalchemy.interfaces


class BaseQuery(sw.allotmentclub.browser.base.Query):

    def select(self):
        return (
            self.db.query(
                Event.id.label('#'),
                Event.category.label('Liste'),
                Event.title.label('Titel'),
                Event.description.label('Beschreibung'),
                Event.start.label('Start'),
                Event.end.label('Ende'),
                to_string('Termin').label('Typ'),
                Event.allday.label('Ganztag'),
                User.username.label('Owner'),
            )
            .select_from(Event)
            .join(User)
            .filter(Event.start.isnot(None))
            .filter(Event.title.isnot(None))
            .filter(Event.organization_id == self.user.organization_id)
        )


class VorstandQuery(BaseQuery):

    def select(self):
        query = super(VorstandQuery, self).select()
        query = query.filter(Event.category == 'Vorstand')

        sitzungen = (
            self.db.query(
                sqlalchemy.null().label('#'),
                to_string('Vorstand').label('Liste'),
                Protocol.title.label('Titel'),
                Protocol.location.label('Beschreibung'),
                Protocol.day.label('Start'),
                (
                    Protocol.day + sqlalchemy.func.cast(
                        concat(3, ' HOURS'), INTERVAL
                    )
                ).label('Ende'),
                to_string('Vorstandssitzung').label('Typ'),
                sqlalchemy.false().label('Ganztag'),
                to_string('').label('Owner'),
            )
            .select_from(Protocol)
            .filter(Protocol.day.isnot(None))
            .filter(Protocol.organization_id == self.user.organization_id)
        )

        return query.union(sitzungen)


class MitgliederQuery(BaseQuery):

    def select(self):
        query = super(MitgliederQuery, self).select()
        query = query.filter(Event.category == 'Mitglieder')

        arbeitseinsaetze = (
            self.db.query(
                sqlalchemy.null().label('#'),
                to_string('Mitglieder').label('Liste'),
                Assignment.purpose.label('Titel'),
                to_string(Member.lastname).concat(', ').concat(
                    Member.firstname).label('Beschreibung'),
                Assignment.day.label('Start'),
                (
                    Assignment.day + sqlalchemy.func.cast(
                        concat(2.5, ' HOURS'), INTERVAL
                    )
                ).label('Ende'),
                to_string('Arbeitseinsatz').label('Typ'),
                sqlalchemy.false().label('Ganztag'),
                to_string('').label('Owner'),
            )
            .select_from(Assignment)
            .filter(Assignment.day.isnot(None))
            .filter(Assignment.organization_id == self.user.organization_id)
            .join(Member)
        )

        lastschriften = (
            self.db.query(
                sqlalchemy.null().label('#'),
                to_string('Mitglieder').label('Liste'),
                to_string('Einzug ').concat(BookingKind.title).label('Titel'),
                to_string(
                    'Bitte sorgen Sie für ausreichend Deckung auf Ihrem '
                    'Konto').label('Beschreibung'),
                SEPASammler.booking_day.label('Start'),
                SEPASammler.booking_day.label('Ende'),
                to_string('Lastschrift').label('Typ'),
                sqlalchemy.true().label('Ganztag'),
                to_string('').label('Owner'),
            )
            .select_from(SEPASammler)
            .filter(SEPASammler.booking_day.isnot(None))
            .filter(SEPASammler.organization_id == self.user.organization_id)
            .join(BookingKind)
        )

        return query.union(arbeitseinsaetze).union(lastschriften)


class BirthdayQuery(sw.allotmentclub.browser.base.Query):

    def select(self, year=None):
        if not year:
            year = get_selected_year()
        return (
            self.db.query(
                sqlalchemy.null().label('#'),
                to_string('Geburtstage').label('Liste'),
                to_string(Member.lastname).concat(', ').concat(
                    Member.firstname).label('Titel'),
                (
                    to_string(year - sqlalchemy.cast(
                        sqlalchemy.func.extract('year', Member.birthday),
                        sqlalchemy.INTEGER
                    )).concat('. Geburtstag')
                ).label('Beschreibung'),
                sqlalchemy.cast((
                    to_string(year)
                    .concat('-')
                    .concat(
                        sqlalchemy.func.extract('month', Member.birthday)
                    )
                    .concat('-')
                    .concat(
                        sqlalchemy.func.extract('day', Member.birthday)
                    )
                ), sqlalchemy.DATE).label('Start'),
                sqlalchemy.null().label('Ende'),
                to_string('Geburtstag').label('Typ'),
                sqlalchemy.true().label('Ganztag'),
                to_string('').label('Owner'),
            )
            .select_from(Member)
            .filter(Member.birthday.isnot(None))
            .filter(Member.leaving_year.is_(None))
            .filter(Member.organization_id == self.user.organization_id)
        )


class Query(sw.allotmentclub.browser.base.Query):

    def select(self):
        query = VorstandQuery(self.db, self.user).select().union(
            MitgliederQuery(self.db, self.user).select())
        year = get_selected_year()
        for year in [year-1, year, year+1]:
            query = query.union(BirthdayQuery(self.db, self.user).select(year))
        return query.distinct()


@view_config(route_name='calendar', renderer='json', permission='view')
class CalendarView(sw.allotmentclub.browser.base.TableView):

    query_class = Query

    available_actions = [
        dict(url='infobrief_print', btn_class='btn-success',
             icon='fa fa-print', title='Termine Info-Brief')]


@view_config(route_name='infobrief_print', permission='view')
class InfobriefPrintView(sw.allotmentclub.browser.base.PrintBaseView):

    filename = 'Termine'
    subject = 'Termine'
    message = """
<h2>Lastschrifteinzüge</h2>
<table style="font-size: 10pt;">
  <tbody>
    {{#each Lastschrift}}
    <tr>
      <td style="width: 40%;">{{date}}</td>
      <td style="width: 60%">{{title}}</td>
    </tr>
    {{/each}}
  </tbody>
</table>

<h2>wichtige Termine</h2>
<table style="font-size: 10pt;">
  <tbody>
    {{#each Termin}}
    <tr>
      <td style="width: 40%;">{{date}}</td>
      <td style="width: 60%">{{title}}</td>
    </tr>
    {{/each}}
  </tbody>
</table>

<h2>Geplante Arbeitseinsätze</h2>
<table style="font-size: 10pt;">
  <tbody>
    {{#each Arbeitseinsatz}}
    <tr>
      <td style="width: 40%;">{{date}}</td>
      <td style="width: 60%">{{title}}</td>
    </tr>
    {{/each}}
  </tbody>
</table>"""

    def get_pdf(self):
        year = get_selected_year()
        subject = self.subject + f' {year}'
        compiler = pybars.Compiler()
        template = compiler.compile(self.message)

        db = zope.component.getUtility(
            risclog.sqlalchemy.interfaces.IDatabase
        )
        query = MitgliederQuery(db, self.request.user)
        events = []
        for event in query.select():
            if event[4].year != year:
                continue
            events.append(
                dict(
                    date=event[4],
                    title=event[2],
                    type_=event[-3],
                    allday=event[-2]
                )
            )
        data = {
            'Lastschrift': [],
            'Arbeitseinsatz': [],
            'Termin': [],
        }
        for event in sorted(events, key=lambda x: x['date']):
            datum = (
                date(event['date']) if event['allday']
                else date_time(event['date'])
            )
            data[event['type_']].append(dict(
                date=datum, title=event['title']
            ))

        message = "".join(template(data))
        return render_pdf(None, subject, message, self.request.user)


@view_config(route_name='calendar_event_add', renderer='json',
             permission='view')
class CalendarEventAddView(sw.allotmentclub.browser.base.View):

    model = Event

    def __call__(self):
        data = self.request.json
        event = Event.create(user=self.request.user)
        event.allday = data['allday']
        if event.allday:
            offset = 10
        else:
            offset = 16
        event.start = data['start'][0:offset]
        if data['end']:
            event.end = data['end'][0:offset]
        event.category = data['category']
        event.title = data['title']
        event.description = data['description']
        event.commit()

        log_with_user(
            user_data_log.info,
            self.request.user,
            f'Termin {event.id} ({event.title}) in Kalender eingefügt.'
        )
        return event.id

    def log(self):
        if self.deleted is None:
            return
        if self.context.user != pyramid.threadlocal.get_current_request().user:
            raise RuntimeError('Not allowed to delete event')
        log_with_user(
            user_data_log.info,
            self.request.user,
            f'Termin {self.context.id} ({self.context.title}) aus Kalender '
            f'gelöscht.'
        )


@view_config(route_name='calendar_event_delete', renderer='json',
             permission='view')
class CalendarEventDeleteView(sw.allotmentclub.browser.base.DeleteView):

    model = Event

    def log(self):
        if self.deleted is None:
            return
        if self.context.user != pyramid.threadlocal.get_current_request().user:
            raise RuntimeError('Not allowed to delete event')
        log_with_user(
            user_data_log.info,
            self.request.user,
            f'Termin {self.context.id} ({self.context.title}) aus Kalender '
            f'gelöscht.'
        )



# encoding=utf8
"""Add new bookingkinds.

Revision ID: 126c6a200c9f
Revises: 32b7b30817d6
Create Date: 2015-11-18 13:04:39.964719

"""
from __future__ import unicode_literals
from alembic import op


# revision identifiers, used by Alembic.
revision = '126c6a200c9f'
down_revision = u'32b7b30817d6'


def upgrade():
    op.execute("""INSERT into bookingkind (title, shorttitle) VALUES
                  ('Grundbesitzabgabe', 'GRUN')""")
    op.execute("""INSERT into bookingkind (title, shorttitle) VALUES
                  ('Versicherungen', 'VERS')""")
    op.execute("""INSERT into bookingkind (title, shorttitle) VALUES
                  ('Verwaltung', 'VERW')""")
    op.execute("""INSERT into bookingkind (title, shorttitle) VALUES
                  ('Aufwandsentschädigungen', 'AUFW')""")
    op.execute("""INSERT into bookingkind (title, shorttitle) VALUES
                  ('Instandhaltung', 'INST')""")
    op.execute("""INSERT into bookingkind (title, shorttitle) VALUES
                  ('Energieanbieter', 'ENAN')""")
    op.execute("""INSERT into bookingkind (title, shorttitle) VALUES
                  ('Müll', 'MÜLL')""")
    op.execute("""INSERT into bookingkind (title, shorttitle) VALUES
                  ('Vereinsfest', 'VEFE')""")
    op.execute("""INSERT into bookingkind (title, shorttitle) VALUES
                  ('Pacht', 'PACH')""")


def downgrade():
    pass



# -* -coding: utf-8 -*-
from ..log import app_log, error_log, auth_log, log_with_user
from pyramid.authentication import SessionAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy
from pyramid.events import NewRequest
from pyramid.security import NO_PERMISSION_REQUIRED
import hashlib
import json
import mimetypes
import pkg_resources
import pyramid.config
import pyramid.events
import pyramid.httpexceptions
import pyramid.i18n
import pyramid.settings
import pyramid.threadlocal
import pyramid.view
import risclog.sqlalchemy.serializer
import signal
import sw.allotmentclub
import sw.allotmentclub.application
import sw.allotmentclub.browser
import sw.allotmentclub.browser.auth
import sw.allotmentclub.browser.base
import sw.allotmentclub.version
import time
import traceback
import zope.component
import sentry_sdk
from sentry_sdk.integrations.pyramid import PyramidIntegration
from sentry_sdk.integrations.sqlalchemy import SqlalchemyIntegration


def json_serializer(value, default, **kw):
    return json.dumps(value, default=default, **kw)


def check_csrf(request):
    token = request.session.get_csrf_token()
    csrf_token = request.POST.get('csrf_token')
    return token == str(csrf_token)


class WSGIWrapper(object):

    def __init__(self, app, **settings):
        self.app = app
        self.settings = settings

    def __call__(self, environ, start_response):
        def start_response_wrapper(status, headers, exc_info=None):
            self.mangle_headers(headers)
            return start_response(status, headers, exc_info)
        return self.app(environ, start_response_wrapper)


class DenyFrame(WSGIWrapper):

    def mangle_headers(self, headers):
        headers.append(('X-Frame-Options', 'SAMEORIGIN'))


class NoCache(WSGIWrapper):

    def mangle_headers(self, headers):
        headers.append(('cache-control', 'no-cache'))
        headers.append(('cache-control', 'no-store'))
        headers.append(('pragma', 'no-cache'))


class Portal(sw.allotmentclub.application.Application):
    """The portal application."""

    def __call__(self, global_config, **settings):
        super(Portal, self).__call__(**settings)
        app = self.make_wsgi_app(global_config)
        return app

    def configure(self):
        super(Portal, self).configure()
        self.configure_zca()

        mimetypes.add_type('application/font-woff', '.woff')
        mimetypes.add_type('application/x-font-ttf', '.ttf')

        registry = pyramid.registry.Registry(
            bases=(zope.component.getGlobalSiteManager(),))
        if self.settings.get('sentry.dsn', None):
            version = sw.allotmentclub.version.__version__
            sentry_sdk.init(
                release=f"sw-allotmentclub-backend@{version}",
                dsn=self.settings['sentry.dsn'],
                integrations=[PyramidIntegration(), SqlalchemyIntegration()])
        self.config = config = pyramid.config.Configurator(
            settings=self.settings,
            registry=registry)
        config.setup_registry(settings=self.settings)
        config.include('pyramid_beaker')
        config.include('pyramid_exclog')
        config.include('pyramid_tm')

        if self.testing:
            config.include('pyramid_mailer.testing')
        elif config.registry.settings.get('mail.enable') == 'false':
            config.include('sw.allotmentclub.printmailer')
        else:
            config.include('sw.allotmentclub.logmailer')

        # self.add_csrf_check()

        config.add_renderer(
            'json', risclog.sqlalchemy.serializer.json_renderer_factory(
                serializer=json_serializer))

        config.set_default_permission('view')
        config.set_authentication_policy(SessionAuthenticationPolicy())
        config.set_authorization_policy(ACLAuthorizationPolicy())
        config.set_root_factory(
            sw.allotmentclub.browser.auth.get_default_context)
        config.add_request_method(
            sw.allotmentclub.browser.auth.get_user, 'user', property=True)

        self.add_routes()
        config.scan(package=sw.allotmentclub.browser,
                    ignore=sw.allotmentclub.SCAN_IGNORE_TESTS)

    def configure_zca(self):
        pass

    def add_csrf_check(self):
        def check_request_for_csrf(event):
            request = event.request
            if request.POST and not check_csrf(request):
                # disable csrf check for login
                if request.path == '/login':
                    return
                raise sw.allotmentclub.browser.interfaces.CSRFForbidden()
        self.config.add_subscriber(check_request_for_csrf, NewRequest)

    def add_routes(self):
        config = self.config
        config.add_route('sentry_frontend', '/sentry_frontend')
        config.add_route('login', '/login')
        config.add_route('logout', '/logout')

        config.add_route('navigation', '/navigation')

        config.add_route('home', '/')
        config.add_route('map', '/map')
        config.add_route('infobrief_print', '/infobrief_print')
        config.add_route('calendar', '/calendar')
        config.add_route('calendar_event_add', '/calendar/add')
        config.add_route(
            'calendar_event_delete',
            '/calendar/{id}/delete',
            factory='..model.Event.context_factory'
        )
        config.add_route('map_download', '/map/download')
        config.add_route('parcel_list', '/parcels')
        config.add_route('parcel_map_upload', '/parcels/{id}/upload_map',
                         factory='..model.Parcel.context_factory')
        config.add_route('parcel_map_download_check',
                         '/parcels/{id}/check_download_map',
                         factory='..model.Parcel.context_factory')
        config.add_route('parcel_map_download', '/parcels/{id}/download_map',
                         factory='..model.Parcel.context_factory')

        config.add_route('depot', '/depot')

        config.add_route('mail_add', '/mail/add')
        config.add_route('mail_list_inbox', '/mail/inbox')
        config.add_route('mail_list_sent', '/mail/sent')
        config.add_route('mail_list_drafts', '/mail/drafts')
        config.add_route('mail_edit', '/mail/{id}/edit',
                         factory='..mail.Message.context_factory')
        config.add_route('mail_reply', '/mail/{id}/reply',
                         factory='..mail.Message.context_factory')
        config.add_route('mail_status', '/mail/{id}/status',
                         factory='..mail.Message.context_factory')
        config.add_route('mail_preview', '/mail/{id}/preview',
                         factory='..mail.Message.context_factory')
        config.add_route('mail_print', '/mail/{id}/download',
                         factory='..mail.Message.context_factory')
        config.add_route('mail_send', '/mail/{id}/send',
                         factory='..mail.Message.context_factory')
        config.add_route('mail_duplicate', '/mail/{id}/duplicate',
                         factory='..mail.Message.context_factory')
        config.add_route('mail_delete', '/mail/{id}/delete',
                         factory='..mail.Message.context_factory')
        config.add_route('mail_list_attachments', '/mail/{id}/attachments',
                         factory='..mail.Message.context_factory')
        config.add_route('mail_attachment_del', '/mail/attachment/{id}/del',
                         factory='..mail.Attachment.context_factory')
        config.add_route('mail_attachment_white_page',
                         '/mail/attachment/{id}/change_white_page',
                         factory='..mail.Attachment.context_factory')
        config.add_route('mail_attachment_download',
                         '/mail/attachment/{id}/download',
                         factory='..mail.Attachment.context_factory')
        config.add_route('mail_postmark_open_tracking_webhook',
                         '/mail/postmark_open_tracking_webhook')
        config.add_route('mail_postmark_inbound_webhook',
                         '/mail/postmark_inbound_webhook')

        config.add_route('member_assignments', '/members/assignment_attendees')
        config.add_route('member_assignments_bill',
                         '/members/assignment_attendees/bill')
        config.add_route('member_assignments_detail',
                         '/members/assignment_attendees/{id}/list',
                         factory='..model.Member.context_factory')
        config.add_route('member_add', '/members/add')
        config.add_route('member_edit',
                         '/members/{id}/edit',
                         factory='..model.Member.context_factory')
        config.add_route('member_attachment_add',
                         '/members/{id}/attachments/add',
                         factory='..model.Member.context_factory')
        config.add_route('member_attachment', '/members/{id}/attachments',
                         factory='..model.Member.context_factory')
        config.add_route(
            'member_attachment_download',
            '/members/{member_id}/attachments/{id}/download',
            factory='..model.MemberAttachment.context_factory')
        config.add_route(
            'member_attachment_delete',
            '/members/{member_id}/attachments/{id}/delete',
            factory='..model.MemberAttachment.context_factory')
        config.add_route('member_account_list', '/members/account_list')
        config.add_route('member_account_detail_list',
                         '/members/{id}/account_details',
                         factory='..model.Member.context_factory')
        config.add_route(
            'member_account_detail_switch_ir',
            '/members/{member_id}/account_details/{id}/switch',
            factory='.account.AccountDetailFactory')
        config.add_route('banking_account_list', '/accounts/list')
        config.add_route('banking_account_list_detail',
                         '/accounts/{id}/detail',
                         factory='..account.BookingKind.context_factory')
        config.add_route('banking_account_list_report', '/accounts/report.pdf')
        config.add_route('sepa_sammler_list', '/accounts/sepa_sammler')
        config.add_route('sepa_sammler_add', '/accounts/sepa_sammler/add')
        config.add_route(
            'sepa_ueberweisung_add',
            '/accounts/sepa_sammler/add_ueberweisung'
        )
        config.add_route('sepa_sammler_edit',
                         '/accounts/sepa_sammler/{id}/edit',
                         factory='..account.SEPASammler.context_factory')
        config.add_route('sepa_sammler_export',
                         '/accounts/sepa_sammler/{id}/export',
                         factory='..account.SEPASammler.context_factory')
        config.add_route('sepa_sammler_update',
                         '/accounts/sepa_sammler/{id}/update',
                         factory='..account.SEPASammler.context_factory')
        config.add_route('sepa_sammler_entry_list',
                         '/accounts/sepa_sammler/{id}/entries',
                         factory='..account.SEPASammler.context_factory')
        config.add_route('sepa_direct_debit', '/accounts/sepa_direct_debit')
        config.add_route('booking_list', '/accounts/{id}/list',
                         factory='..account.BankingAccount.context_factory')
        config.add_route('split_booking', '/booking/{id}/split',
                         factory='..account.Booking.context_factory')
        config.add_route('map_booking', '/booking/{id}/map',
                         factory='..account.Booking.context_factory')

        config.add_route('waste_water', '/waste_water')
        config.add_route('property_tax_b', '/property_tax_b')

        config.add_route('member_list', '/members')
        config.add_route('member_list_leased', '/members_leased')
        config.add_route('member_list_passive', '/members_passive')
        config.add_route('member_list_tap_water', '/members_tap_water')
        config.add_route('member_sale_history', '/members/sale_history')
        config.add_route('membership_fee', '/members/insert_membership_fee')
        config.add_route('member_sale',
                         '/members/{id}/sale',
                         factory='..model.Member.context_factory')
        config.add_route('direct_debit_letter',
                         '/members/{id}/direct_debit_letter',
                         factory='..model.Member.context_factory')
        config.add_route('become_member_letter',
                         '/members/{id}/become_member_letter',
                         factory='..model.Member.context_factory')
        config.add_route('mv_entrance_list', '/members/mv_entrance_list')

        config.add_route('energy_meter_export', '/electricity/export')
        config.add_route('energy_meter_import', '/electricity/import')
        config.add_route('calculate_energy_values',
                         '/electricity/calculate_energy_values')

        config.add_route('access_authority', '/access_authority')
        config.add_route('access_authority_detail',
                         '/access_authority/{id}/list',
                         factory='.authority.AuthorityContext.context_factory')
        config.add_route('access_authority_detail_add',
                         '/access_authority/{id}/list/add',
                         factory='.authority.AuthorityContext.context_factory')
        config.add_route(
            'access_authority_detail_edit',
            '/access_authority/{viewname}/list/{id}/edit',
            factory='..model.AccessAuthority.context_factory')
        config.add_route(
            'access_authority_detail_delete',
            '/access_authority/{viewname}/list/{id}/delete',
            factory='..model.AccessAuthority.context_factory')

        config.add_route('energy_price', '/energy_price')
        config.add_route('electricity_list', '/electricity')
        config.add_route('global_energy_value_list', '/electricity_billing')
        config.add_route('energy_value_list',
                         '/electricity/{id}/history',
                         factory='..electricity.ElectricMeter.context_factory')
        config.add_route('advance_pay_value_list',
                         '/electricity/{id}/advance_pay_history',
                         factory='..electricity.ElectricMeter.context_factory')

        config.add_route('externals', '/externals')
        config.add_route('external_add', '/externals/add')
        config.add_route('external_edit', '/externals/{id}/edit',
                         factory='..mail.ExternalRecipient.context_factory')
        config.add_route('external_delete', '/externals/{id}/delete',
                         factory='..mail.ExternalRecipient.context_factory')

        config.add_route('bulletins', '/bulletins')
        config.add_route('bulletin_add', '/bulletins/add')
        config.add_route('bulletin_edit', '/bulletins/{id}/edit',
                         factory='..bulletins.Bulletin.context_factory')
        config.add_route('bulletin_delete', '/bulletins/{id}/delete',
                         factory='..bulletins.Bulletin.context_factory')
        config.add_route('bulletin_print', '/bulletins/{id}/print',
                         factory='..bulletins.Bulletin.context_factory')

        config.add_route('keylists', '/keylists')
        config.add_route('keylist_add', '/keylists/add')
        config.add_route('keylist_edit', '/keylists/{id}/edit',
                         factory='..keylist.Keylist.context_factory')
        config.add_route('keylist_delete', '/keylists/{id}/delete',
                         factory='..keylist.Keylist.context_factory')
        config.add_route('keys', '/keylists/{id}/keys',
                         factory='..keylist.Keylist.context_factory')
        config.add_route('key_add', '/keylists/{id}/keys/add',
                         factory='..keylist.Keylist.context_factory')
        config.add_route(
            'key_edit',
            '/keylists/{keylist_id}/keys/{id}/edit',
            factory='..keylist.Key.context_factory')
        config.add_route(
            'key_delete',
            '/keylists/{keylist_id}/keys/{id}/delete',
            factory='..keylist.Key.context_factory')
        config.add_route('keylist_attachment_add',
                         '/keylists/{id}/attachments/add',
                         factory='..keylist.Keylist.context_factory')
        config.add_route('keylist_attachment', '/keylists/{id}/attachments',
                         factory='..keylist.Keylist.context_factory')
        config.add_route(
            'keylist_attachment_download',
            '/keylists/{keylist_id}/attachments/{id}/download',
            factory='..keylist.KeylistAttachment.context_factory')

        config.add_route('protocols', '/protocols')
        config.add_route('protocol_add', '/protocols/add')
        config.add_route('protocol_edit', '/protocols/{id}/edit',
                         factory='..protocol.Protocol.context_factory')
        config.add_route('protocol_delete', '/protocols/{id}/delete',
                         factory='..protocol.Protocol.context_factory')
        config.add_route('protocol_detail', '/protocols/{id}/details',
                         factory='..protocol.Protocol.context_factory')
        config.add_route('protocol_detail_add', '/protocols/{id}/details/add',
                         factory='..protocol.Protocol.context_factory')
        config.add_route(
            'protocol_detail_edit',
            '/protocols/{protocol_id}/details/{id}/edit',
            factory='..protocol.ProtocolDetail.context_factory')
        config.add_route(
            'protocol_detail_delete',
            '/protocols/{protocol_id}/details/{id}/delete',
            factory='..protocol.ProtocolDetail.context_factory')
        config.add_route('protocol_print', '/protocols/{id}/print',
                         factory='..protocol.Protocol.context_factory')
        config.add_route('protocol_attachment_add',
                         '/protocols/{id}/attachments/add',
                         factory='..protocol.Protocol.context_factory')
        config.add_route('protocol_attachment', '/protocols/{id}/attachments',
                         factory='..protocol.Protocol.context_factory')
        config.add_route(
            'protocol_attachment_download',
            '/protocols/{protocol_id}/attachments/{id}/download',
            factory='..protocol.ProtocolAttachment.context_factory')
        config.add_route(
            'protocol_attachment_delete',
            '/protocols/{protocol_id}/attachments/{id}/delete',
            factory='..protocol.ProtocolAttachment.context_factory')
        config.add_route('protocol_commitment', '/protocols/{id}/commitments',
                         factory='..protocol.Protocol.context_factory')
        config.add_route('protocol_commitment_add',
                         '/protocols/{id}/commitments/add',
                         factory='..protocol.Protocol.context_factory')
        config.add_route(
            'protocol_commitment_edit',
            '/protocols/{protocol_id}/commitments/{id}/edit',
            factory='..protocol.ProtocolCommitment.context_factory')
        config.add_route(
            'protocol_commitment_delete',
            '/protocols/{protocol_id}/commitments/{id}/delete',
            factory='..protocol.ProtocolCommitment.context_factory')

        config.add_route('assignments', '/assignments')
        config.add_route('assignment_add', '/assignments/add')
        config.add_route('assignment_edit', '/assignments/{id}/edit',
                         factory='..assignment.Assignment.context_factory')
        config.add_route('assignment_delete', '/assignments/{id}/delete',
                         factory='..assignment.Assignment.context_factory')
        config.add_route('assignment_list_attendees', '/assignments/{id}/list',
                         factory='..assignment.Assignment.context_factory')
        config.add_route('assignment_attendees_add',
                         '/assignments/{id}/attendees/add',
                         factory='..assignment.Assignment.context_factory')
        config.add_route(
            'assignment_attendees_edit',
            '/assignments/{assignment_id}/attendees/{id}/edit',
            factory='..assignment.AssignmentAttendee.context_factory')

        config.add_route(
            'assignment_attendees_delete',
            '/assignments/{assignment_id}/attendees/{id}/delete',
            factory='..assignment.AssignmentAttendee.context_factory')

        config.add_route('depots', '/depots')
        config.add_route('depot_add', '/depots/add')
        config.add_route('depot_edit', '/depots/{id}/edit',
                         factory='..depot.Depot.context_factory')
        config.add_route('depot_delete', '/depots/{id}/delete',
                         factory='..depot.Depot.context_factory')
        config.add_route('depot_download', '/depots/{id}/download',
                         factory='..depot.Depot.context_factory')

    @property
    def pipeline(self):
        return [
            (DenyFrame, 'factory', None, {}),
            (NoCache, 'factory', None, {}),
        ]

    def make_wsgi_app(self, global_config):
        app = self.config.make_wsgi_app()
        for spec, protocol, name, extra in self.pipeline:
            if protocol == 'factory':
                app = spec(app, **extra)
                continue
            entrypoint = pkg_resources.get_entry_info(spec, protocol, name)
            app = entrypoint.load()(app, global_config, **extra)
        return app


factory = Portal()


@pyramid.view.view_config(
    context=Exception,
    permission=NO_PERMISSION_REQUIRED,
    renderer='json')
class Error(sw.allotmentclub.browser.base.View):

    title = u'Interner Server-Fehler'
    status_code = 500

    def __init__(self, context, request):
        super(Error, self).__init__(context, request)
        self.time = time.time()

    @property
    def show_traceback(self):
        return self.request.registry.settings['testing']

    @property
    def exc_line(self):
        return traceback.format_exception_only(
            type(self.context), self.context)[0].strip()

    @property
    def error_ref(self):
        return (str(int(self.time))[-4:] +
                str(int(hashlib.sha1(
                    self.tb.encode('utf-8')).hexdigest(), 16))[:4])

    def update(self):
        self.tb = traceback.format_exc()
        self.error_summary = ('Ref: %s %s', self.error_ref, self.exc_line)
        self.request.response.status_code = self.status_code
        self.log()

    def log(self):
        error_log.error(*self.error_summary)
        for line in self.tb.splitlines():
            error_log.info(line, log_request=False)
        if self.show_traceback:
            print('Exception raised during test: {}'.format(self.tb))


@pyramid.view.view_config(
    route_name='sentry_frontend',
    permission=NO_PERMISSION_REQUIRED,
    renderer='json')
class SentryDataView(sw.allotmentclub.browser.base.View):

    def __call__(self):
        settings = pyramid.threadlocal.get_current_registry().settings
        return dict(dsn=settings.get('sentry.frontenddsn', ''))


@pyramid.view.view_config(
    context=pyramid.httpexceptions.HTTPException,
    permission=NO_PERMISSION_REQUIRED)
def HTTPException(context, request):
    return context


@pyramid.view.view_config(
    context=pyramid.httpexceptions.HTTPError,
    permission=NO_PERMISSION_REQUIRED,
    renderer='json')
class HTTPError(Error):

    @property
    def title(self):
        return '%s %s' % (self.status_code, type(self.context).__name__)

    @property
    def status_code(self):
        return self.context.status_code


@pyramid.view.notfound_view_config(renderer='json')
class NotFound(HTTPError):

    def log(self):
        if self.request.user.authenticated:
            log_with_user(
                auth_log.warning, self.request.user,
                'Ref: %s User %s tried to access non-existing url %s',
                self.error_ref, self.request.user.username, self.request.url)
        else:
            log_with_user(
                auth_log.warning, self.request.user,
                'Ref: %s Unauthenticated user tried to access '
                'non-existing url %s', self.error_ref, self.request.url)


@pyramid.events.subscriber(pyramid.events.ApplicationCreated)
def log_start(event):
    app_log.info('Application successfully started.')


shutdown_already_logged = False


def log_shutdown(signalnum, frame):
    global shutdown_already_logged
    if not shutdown_already_logged:
        shutdown_already_logged = True
        app_log.info('Application shutdown.')
    if signalnum == signal.SIGINT:
        # Make sure the port is closed, too:
        orig_sigint_handler(signalnum, frame)


signal.signal(signal.SIGTERM, log_shutdown)
orig_sigint_handler = signal.signal(signal.SIGINT, log_shutdown)



# encoding: utf-8
from __future__ import unicode_literals
from .navigation import NAVIGATION_ITEMS
from .base import boolean, get_view_for_route
from pyramid.view import view_config
from sw.allotmentclub import User, AccessAuthority
import sw.allotmentclub.browser.base
import collections


class AuthorityContext(object):

    id = None

    @classmethod
    def context_factory(cls, request):
        import sw.allotmentclub.browser.auth
        result = cls()
        result.id = request.matchdict['id']
        result.__acl__ = sw.allotmentclub.browser.auth.authorize()
        return result


@view_config(route_name='access_authority', renderer='json',
             permission='view')
class AccessAuthorityListView(sw.allotmentclub.browser.base.View):
    """ Liste aller Routen für die Berechtigungsaussteuerung."""

    def add_sub_actions(self, items, id):
        parent = id
        view = get_view_for_route(id.replace('_group', ''), self.request)
        for act in getattr(
                view.__original_view__, 'available_actions', []):
            items.append(
                (act['url'], act['title'], parent, {}, act['icon']))
            items = self.add_sub_actions(items, act['url'])
        return items

    def update(self):
        data = []
        items = []  # id, text, parent, state, icon
        for text, id, icon, children in NAVIGATION_ITEMS:
            parent = '#'
            if id == 'access_authority':
                continue
            state = dict()
            if children is not None:
                state['disabled'] = True
            items.append((id, text, parent, state, icon))
            if 'disabled' not in state:
                items = self.add_sub_actions(items, id)
            parent = id
            if children is None:
                continue
            if 'disabled' in state:
                parent = parent + '_group'
            for text, id in children:
                items.append((id, text, parent, {}, icon))
                items = self.add_sub_actions(items, id)

        for id, text, parent, state, icon in items:
            if 'disabled' in state:
                id = id + '_group'
            if 'glyphicon' not in icon:
                icon = 'fa fa-lg fa-fw {}'.format(icon)
            data.append(
                dict(id=id, text=text, parent=parent, state=state, icon=icon))
        self.result = dict(data=data, status='success')


class Query(sw.allotmentclub.browser.base.Query):

    formatters = {
        'Gesperrt': boolean,
    }

    def select(self):
        return (
            self.db.query(
                AccessAuthority.id.label('#'),
                User.username.label('User'),
                User.nachname.label('Nachname'),
                User.vorname.label('Vorname'),
                User.is_locked.label('Gesperrt'))
            .select_from(AccessAuthority)
            .join(User)
            .filter(AccessAuthority.viewname == self.context.id))


@view_config(route_name='access_authority_detail', renderer='json',
             permission='view')
class AccessAuthorityDetailsView(sw.allotmentclub.browser.base.TableView):
    """Liste aller Zugriffsberechtigungen für einen View."""

    query_class = Query
    available_actions = [
        dict(url='access_authority_detail_add', btn_class='btn-success',
             icon='fa fa-plus', title='Neu'),
        dict(url='access_authority_detail_delete', btn_class='btn-danger',
             icon='glyphicon glyphicon-trash', title='Löschen')]


@view_config(route_name='access_authority_detail_edit', renderer='json',
             permission='view')
class AccessAuthorityDetailEditView(
        sw.allotmentclub.browser.base.EditJSFormView):

    title = 'Berechtigung hinzufügen'

    @property
    def users(self):
        query = super(AccessAuthorityDetailEditView, self).users
        query = query.filter(User.unrestricted_access.is_(False))
        return query

    @property
    def load_options(self):
        return {
            'user_id': {
                'label': 'Benutzer',
                'source': self.user_source,
                'css_class': 'chosen',
                'required': True
            },
        }

    @property
    def load_data(self):
        fields = [
            ('user_id', self.context.user_id),
        ]
        return collections.OrderedDict(fields)

    def get_route(self, item, name):
        route = super(AccessAuthorityDetailEditView, self).get_route(
            item, name)
        return route.replace('{viewname}', self.context.viewname)


@view_config(route_name='access_authority_detail_add', renderer='json',
             permission='view')
class AccessAuthorityDetailAddView(AccessAuthorityDetailEditView):

    title = 'Berechtigung bearbeiten'

    def __init__(self, context, request):
        context = AccessAuthority.create(viewname=context.id)
        context.commit()
        super(AccessAuthorityDetailAddView, self).__init__(context, request)

    @property
    def route_name(self):
        return 'access_authority_detail_edit'


@view_config(route_name='access_authority_detail_delete', renderer='json',
             permission='view')
class AccessAuthorityDetailDeleteView(
        sw.allotmentclub.browser.base.DeleteView):

    model = AccessAuthority





def test_no_route_to_energy_meter_import_if_not_logged_in(browser):
    browser.open('http://localhost/navigation')
    assert 'energy_meter_import' not in str(browser.json['data'])


def test_route_to_access_authority_if_logged_in_as_admin(browser):
    browser.login()
    browser.open('http://localhost/navigation')
    assert 'access_authority' in str(browser.json['data'])


def test_no_route_to_access_authority_if_logged_in_as_non_admin(browser):
    import transaction
    from ...user import User
    User.create(
        username='user', password='user',
        unrestricted_access=False)
    transaction.commit()

    browser.login(username='user', password='user')
    browser.open('http://localhost/navigation')
    assert 'access_authority' not in str(browser.json['data'])


def test_route_to_configured_route_if_logged_in_as_non_admin(browser):
    import transaction
    from ...user import User
    from ...model import AccessAuthority
    user = User.create(
        username='user', password='user',
        unrestricted_access=False)
    AccessAuthority.find_or_create(viewname='access_authority', user=user)

    transaction.commit()

    browser.login(username='user', password='user')
    browser.open('http://localhost/navigation')
    assert 'access_authority' in str(browser.json['data'])



"""Prepare message hierachy.

Revision ID: fd9f9588878d
Revises: ff18c87afd8e
Create Date: 2017-06-07 14:34:34.902567

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = 'fd9f9588878d'
down_revision = 'ff18c87afd8e'


def upgrade():
    op.add_column('message',
                  sa.Column('in_reply_to_id', sa.Integer(), nullable=True))
    op.create_foreign_key(
        None, 'message', 'message', ['in_reply_to_id'], ['id'])


def downgrade():
    op.drop_constraint(None, 'message', type_='foreignkey')
    op.drop_column('message', 'in_reply_to_id')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from zope.testbrowser.wsgi import Browser
import json
import sw.allotmentclub


class Browser(Browser):
    """Our enhanced browser."""

    username = 'admin'
    password = 'admin'

    @property
    def json(self):
        return json.loads(self.contents.decode('utf-8'))

    def open(self, url, data=None, xhr=False):
        if xhr:
            self.addXhrHeader()
        super(Browser, self).open(url, data)

    def post(self, url, data, type='json', xhr=False):
        """Type can be

            `json` … convert data to JSON + set content_type
            `plain` … use data as it is."""
        if xhr:
            self.addXhrHeader()
        content_type = None
        if type == 'json':
            content_type = 'application/json'
            data = json.dumps(data)
        elif type != 'plain':
            content_type = type
        super(Browser, self).post(url, data, content_type)

    def get(self, url, xhr=False):
        if xhr:
            self.addXhrHeader()
        super(Browser, self).open(url)

    def _upload(self, url, filedata=None):
        if filedata is None:
            from io import BytesIO
            filedata = (
                'Transparent.gif', 'image/gif',
                BytesIO(
                    b'GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\xff\xff'
                    b'\xff!\xf9\x04\x01\x00\x00\x00\x00,\x00\x00\x00\x00\x01'
                    b'\x00\x01\x00\x00\x02\x01D\x00'))
        self.post(
            url,
            type=(
                'multipart/form-data; boundary='
                '---------------------------13041189051108335053514081693'),
            data=(b'-----------------------------13041189051108335053514081693'
                  b'\r\nContent-Disposition: form-data; name="file"; '
                  b'filename="%b"\r\nContent-type: %b\r\n\r\n%b;\r\n'
                  b'-----------------------------13041189051108335053514081693'
                  b'--') % (filedata[0].encode(),
                            filedata[1].encode(),
                            filedata[2].read()))

    def login(self, username=None, password=None):
        """Log in via browser."""
        if username is None:
            username = self.username
        if password is None:
            password = self.password

        self.post(
            'http://localhost/login',
            'username=%s&password=%s' % (username, password),
            'plain')

    def logout(self):
        self.open('http://localhost/logout')

    def addXhrHeader(self):
        self.addHeader('X_REQUESTED_WITH', 'XMLHttpRequest')

    @property
    def json_result(self):
        return sw.allotmentclub.json_result(self.json['data']['data'])



"""Add message attachment.

Revision ID: 38a169e80b6e
Revises: 4369f16d7e67
Create Date: 2015-11-16 11:38:16.714582

"""

# revision identifiers, used by Alembic.
from alembic import op
import sqlalchemy as sa

revision = '38a169e80b6e'
down_revision = '4369f16d7e67'


def upgrade():
    op.create_table(
        'attachment',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('message_id', sa.Integer(), nullable=False),
        sa.Column('filename', sa.String(length=100), nullable=True),
        sa.Column('mimetype', sa.String(length=30), nullable=True),
        sa.Column('size', sa.String(length=20), nullable=True),
        sa.Column('data', sa.LargeBinary(length=10485760), nullable=True),
        sa.ForeignKeyConstraint(['message_id'], [u'message.id'], ),
        sa.PrimaryKeyConstraint('id')
    )


def downgrade():
    op.drop_table('attachment')
